import passport from 'passport';
import LdapStrategy from 'passport-ldapauth';
import fs from 'fs';
import path from 'path';

/**
 * @description Configuration de passport pour utiliser l'authentification LDAP
 * @author kadabra
 *
 * on a besoin d'authentification pour 2 trucs :
 * - l'acces a l'interface admin (admin_view) du back, definie dans admin_router.js
 * - le contexte graphQL
 * 
 * serializeUser permet d'obtenir une cle identifiant chaque user
 * deserializeUser fait une requete vers une BDD de users en utilisant cette cle, et met dans l'objet JS req.user toutes les infos issues de la BDD
 * Cette repartition permet de ne stocker dans la session (i.e. en memoire sur le serveur) que la cle des utilisateurs connectes et de ne "charger en memoire" toutes les infos de la BDD que lorsque necessaire
 * cf https://stackoverflow.com/questions/27637609/understanding-passport-serialize-deserialize#27637668
 * et http://toon.io/understanding-passportjs-authentication-flow/
 * 
 * Mais en fait dans notre cas c'est graphql qui communique avec la BDD, donc on s'en fiche! On peut se contenter de dire a serializeUser et deserializeUser de ne s'occuper que du champ uid)
 * (on pourrait penser que passer par deserializeUser permettrait de reduire le nombre d'interactions avec la BDD, mais en fait non car deserializeUser est appele *a chaque requete*)
 */
const configPath = path.resolve('./', 'ldap_config.json');
const config = JSON.parse(fs.readFileSync(configPath, 'utf8'));

// specifies options for 'ldapauth' strategy, to customize the behaviour of subsequent passport.authenticate('ldapauth') calls
passport.use(new LdapStrategy({
    server: {
        url: config.ldap.server,
        //bindDn: '.............',
        //bindCredentials: '..........',
        searchBase: config.ldap.searchBase,
        searchFilter: config.ldap.searchFilter,
        //searchAttributes: ['givenName', 'sn'],
        //tlsOptions: '..........',
    },

    //usernameField: 'username', // Field name where the username is found, defaults to username
    //passwordField: 'password', // Field name where the password is found, defaults to password

    // given how LdapStrategy is coded, it is not necessary to do a verify callback
    // https://github.com/vesse/passport-ldapauth/blob/master/lib/passport-ldapauth/strategy.js#L230
    // (note that LdapStrategy has no default verify callback, the "verify" function (L105) is actually the "done" function that is called by the verify callback if we choose to make one)
    // we leave this commented out as a template for future use
    /*
    function (user, done) {
        // "verify callback", called after each passport.authenticate(...),
        // unless missing credentials (in which case a 400 Error is returned)

        // "The purpose of a verify callback is to find the user that possesses a set of credentials" (from passport doc)
        // i.e. we query the database (in our case the LDAP) to get user's data
        console.log("Entering passport's verify callback");

        if (user){
            //if user exists
            console.log("Successfully authenticated " + user.uid);
        }
    }
    */
}));


//toujours bon a savoir pour faire des tests:
//The result of the serializeUser method is attached to the session as req.session.passport.user
passport.serializeUser(function (user, done) {
    console.log(`passport.serializeUser(): serializing user ${user.uid}`); // DEBUG
    done(null, user.uid);
});

//The first argument of deserializeUser corresponds to the key of the user object that was given to the done function in serializeUser
//The fetched object is attached to the request object as req.user (available in all subsequent middleware)
passport.deserializeUser(function (userUid, done) {
    console.log(`passport.deserializeUser(): deserializing user ${userUid}`); // DEBUG
    done(null, { uid: userUid });
});